package com.tanpu.community.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.green.model.v20180509.ImageAsyncScanResultsRequest;
import com.aliyuncs.http.FormatType;
import com.aliyuncs.http.HttpResponse;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.http.ProtocolType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tanpu.common.constant.BizStatus;
import com.tanpu.common.util.AliyunOSSHelper;
import com.tanpu.common.util.JsonUtil;
import com.tanpu.common.uuid.UuidGenHelper;
import com.tanpu.community.api.CommunityConstant;
import com.tanpu.community.api.enums.FileTypeEnum;
import com.tanpu.community.dao.entity.community.FileRecordEntity;
import com.tanpu.community.dao.mapper.community.FileRecordMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Slf4j
@Service
public class OSSFileService {

    @Value("${aliyun.oss.endpoint}")
    public String endpoint;

    @Value("${aliyun.oss.accessId}")
    public String accessId;

    @Value("${aliyun.oss.accessSK}")
    public String accessSK;

    @Value("${aliyun.oss.bucketName}")
    public String bucketName;

    private AliyunOSSHelper ossHelper;

    @Resource
    private FileRecordMapper fileRecordMapper;


    @Autowired
    private UuidGenHelper uuidGenHelper;

    @PostConstruct
    public void init() {
        ossHelper = AliyunOSSHelper.build(endpoint, accessId, accessSK);
    }


    @Transactional
    public FileRecordEntity uploadFile(byte[] data, String fileName, String dirPrefix) {
        String fileSuffix = fileName.substring(fileName.lastIndexOf('.') + 1);
        return uploadFile(data, fileName, fileSuffix, dirPrefix);
    }

    @Transactional
    public FileRecordEntity uploadFile(byte[] data, String fileName, String fileSuffix, String dirPrefix) {
        String[] arr = StringUtils.split(fileName, ".");
        String suffix = arr[arr.length - 1];
        //上传
        String id = uuidGenHelper.getUuidStr();
        String key = CommunityConstant.OSS_PREFIX_FOLDER + dirPrefix + id + "." + suffix;
        ossHelper.writeFile(bucketName, key, data, fileSuffix);

        //落库
        FileRecordEntity record = new FileRecordEntity();
        record.setFileId(uuidGenHelper.getUuidStr());
        record.setDeleteTag(BizStatus.DeleteTag.tag_init);
        record.setFileOssKey(key);
        record.setFileName(fileName);
        record.setFileId(id);
        record.setPreviewUrl(ossHelper.getPreSignedUrl(bucketName, key));
        record.setFileType(FileTypeEnum.IMAGE.getCode());
        HashMap<String, Integer> attr = getStringIntegerHashMap(data);
        record.setExtInfo(JsonUtil.toJson(attr));

        fileRecordMapper.insert(record);

        return record;
    }

    public void uploadFileNoRecord(byte[] data, String fileName, String fileSuffix, String dirPrefix) {
        String[] arr = StringUtils.split(fileName, ".");
        String suffix = arr[arr.length - 1];
        //上传
        String key = CommunityConstant.OSS_PREFIX_FOLDER + dirPrefix + suffix;
        ossHelper.writeFile(bucketName, key, data, fileSuffix);
    }

    private HashMap<String, Integer> getStringIntegerHashMap(byte[] data) {
        ByteArrayInputStream bins = new ByteArrayInputStream(data);
        BufferedImage bi = null;
        try {
            bi = ImageIO.read(bins);
        } catch (Exception e) {
            e.printStackTrace();
        }
        HashMap<String, Integer> attr = new HashMap<>();
        attr.put("width",bi.getWidth());
        attr.put("height",bi.getHeight());
        return attr;
    }

    public FileRecordEntity queryById(String fileId) {
        return fileRecordMapper.selectOne(new LambdaQueryWrapper<FileRecordEntity>()
                .eq(FileRecordEntity::getFileId,fileId));
    }

    public List<FileRecordEntity> queryByIds(List<String> fileIds) {
        return fileRecordMapper.selectList(new LambdaQueryWrapper<FileRecordEntity>()
                .in(FileRecordEntity::getFileId,fileIds));
    }

    public static void main(String[] args) throws Exception {
        IClientProfile profile = DefaultProfile.getProfile("cn-shanghai", "请填写您的accessKeyId", "请填写您的accessKeySecret");
        DefaultProfile.addEndpoint("cn-shanghai", "cn-shanghai", "Green", "green.cn-shanghai.aliyuncs.com");
        IAcsClient client = new DefaultAcsClient(profile);

        ImageAsyncScanResultsRequest imageAsyncScanResultsRequest = new ImageAsyncScanResultsRequest();
        // 指定API返回格式。
        imageAsyncScanResultsRequest.setAcceptFormat(FormatType.JSON);
        // 指定请求方法。
        imageAsyncScanResultsRequest.setMethod(MethodType.POST);
        imageAsyncScanResultsRequest.setEncoding("utf-8");
        // 支持HTTP和HTTPS。
        imageAsyncScanResultsRequest.setProtocol(ProtocolType.HTTP);


        List<String> taskIds = new ArrayList<String>();
        taskIds.add("img4hDosCHcrFk5jAMR80XWJN-1pZ@0p");
        imageAsyncScanResultsRequest.setHttpContent(JSON.toJSONString(taskIds).getBytes("UTF-8"), "UTF-8", FormatType.JSON);

        /**
         * 请务必设置超时时间。
         */
        imageAsyncScanResultsRequest.setConnectTimeout(3000);
        imageAsyncScanResultsRequest.setReadTimeout(6000);

        try {
            HttpResponse httpResponse = client.doAction(imageAsyncScanResultsRequest);

            if(httpResponse.isSuccess()){
                JSONObject scrResponse = JSON.parseObject(new String(httpResponse.getHttpContent(), "UTF-8"));
                System.out.println(JSON.toJSONString(scrResponse, true));
                if (200 == scrResponse.getInteger("code")) {
                    JSONArray taskResults = scrResponse.getJSONArray("data");
                    for (Object taskResult : taskResults) {
                        if(200 == ((JSONObject)taskResult).getInteger("code")){
                            JSONArray sceneResults = ((JSONObject)taskResult).getJSONArray("results");
                            for (Object sceneResult : sceneResults) {
                                String scene = ((JSONObject)sceneResult).getString("scene");
                                String suggestion = ((JSONObject)sceneResult).getString("suggestion");
                                // 根据scene和suggestion做相关的处理。
                                // 根据不同的suggestion结果做业务上的不同处理。例如，将违规数据删除等。
                            }
                        }else{
                            System.out.println("task process fail:" + ((JSONObject)taskResult).getInteger("code"));
                        }
                    }
                } else {
                    System.out.println("detect not success. code:" + scrResponse.getInteger("code"));
                }
            }else{
                System.out.println("response not success. status:" + httpResponse.getStatus());
            }
        } catch (ServerException e) {
            e.printStackTrace();
        } catch (ClientException e) {
            e.printStackTrace();
        } catch (Exception e){
            e.printStackTrace();
        }
    }
}
